﻿// ---------------------------------------------------------------------------
// Simulation einer Pendelschwingung in C++, Variante (a)
// ---------------------------------------------------------------------------
#define _USE_MATH_DEFINES
#include <iostream>
#include <vector>
#include <math.h>
#include "hmdgl.hpp"

// ---------------------------------------------------------------------------
// Die Zustandsvariablen des DGL-Systems werden in Containern des Typs
// <state_type> gespeichert. Als <state_type> sind grundsätzlich alle
// Container geeignet, die die Methoden begin() und end() besitzen.
// ---------------------------------------------------------------------------
typedef std::vector<double> state_type;
const size_t dim = 2;  // Zwei Gleichungen 1. Ordnung

// ---------------------------------------------------------------------------
// Definition des Differentialgleichungssystems
// ---------------------------------------------------------------------------
state_type sys(double /* t */, state_type u)
{
    state_type dudt(dim);
    dudt[0] = u[1];
    dudt[1] = -9.81 / 1.0 * sin(u[0]);
    return dudt;
}

// ---------------------------------------------------------------------------
// Nach jedem Integrationsschritt: Simulationszeit, Pendelauslenkung ausgeben
// ---------------------------------------------------------------------------
void step(double t, state_type u)
{
    std::cout << t << "\t" << 180.0 * u[0] / M_PI << std::endl;
}

// ---------------------------------------------------------------------------
// Hauptprogramm mit Aufruf des numerischen Lösungsverfahrens
// ---------------------------------------------------------------------------
int main()
{
    size_t num_steps = 1000;  // Anzahl der Integrationsschritte
    double t_begin   = 0;     // Startzeitpunkt der Simulation
    double t_end     = 10.0;  // Endzeitpunkt der Simulation

    state_type init(dim);     // Anfangsauslenkung = 45°
    init[0] = 45.0 * M_PI / 180.0;
    init[1] = 0;

    HMDGL::dgl_rk4(t_begin, t_end, num_steps, sys, step, init);
    // HMDGL::dgl_heun(t_begin, t_end, num_steps, sys, step, init);
    // HMDGL::dgl_euler(t_begin, t_end, num_steps, sys, step, init);
}
